package kr.co.vividnext.sodalive.live.reservation

import kr.co.vividnext.sodalive.can.payment.CanPaymentService
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.live.room.LiveRoomRepository
import kr.co.vividnext.sodalive.live.room.LiveRoomType
import kr.co.vividnext.sodalive.member.MemberRepository
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Service
class LiveReservationService(
    private val repository: LiveReservationRepository,
    private val liveRoomRepository: LiveRoomRepository,
    private val memberRepository: MemberRepository,
    private val canPaymentService: CanPaymentService
) {
    fun makeReservation(request: MakeLiveReservationRequest, memberId: Long): MakeLiveReservationResponse {
        val room = liveRoomRepository.findByIdOrNull(id = request.roomId)
            ?: throw SodaException(message = "잘못된 요청입니다.\n다시 시도해 주세요.")

        val member = memberRepository.findByIdOrNull(id = memberId)
            ?: throw SodaException(message = "로그인 정보를 확인해주세요.")

        if (
            room.member!!.id!! != memberId &&
            room.type == LiveRoomType.PRIVATE &&
            (request.password == null || request.password != room.password)
        ) {
            throw SodaException("비밀번호가 일치하지 않습니다.\n다시 확인 후 입력해주세요.")
        }

        if (repository.isExistsReservation(roomId = request.roomId, memberId = memberId)) {
            throw SodaException("이미 예약한 라이브 입니다.")
        }

        val haveCan = member.getChargeCan(request.container) + member.getRewardCan(request.container)
        if (haveCan < room.price) {
            throw SodaException("${room.price - haveCan}캔이 부족합니다. 충전 후 이용해 주세요.")
        }

        if (room.price > 0) {
            canPaymentService.spendCan(
                memberId = member.id!!,
                needCan = room.price,
                canUsage = CanUsage.LIVE,
                liveRoom = room,
                container = request.container
            )
        }

        val reservation = LiveReservation()
        reservation.room = room
        reservation.member = member
        repository.save(reservation)

        val beginDateTime = room.beginDateTime
            .atZone(ZoneId.of("UTC"))
            .withZoneSameInstant(ZoneId.of(request.timezone))

        return MakeLiveReservationResponse(
            reservationId = reservation.id!!,
            nickname = room.member!!.nickname,
            title = room.title,
            beginDateString = beginDateTime.format(DateTimeFormatter.ofPattern("yyyy년 M월 d일 (E), a hh:mm")),
            price = if (room.price > 0) {
                "${room.price} 캔"
            } else {
                "무료"
            },
            haveCan = haveCan,
            useCan = room.price,
            remainingCan = haveCan - room.price
        )
    }
}
